from ._TableError import TableError


class TableNotOpened(TableError):
    def __init__(
        self,           # type: TableNotOpened
        table_id,       # type: str
    ):
        self.table_id = table_id

    def get_extra_data(self):
        return {
            "tableId": self.table_id,
        }
