INVALID_JSON = (-1, "The request must be a valid json")
EVENT_SUBJECT_MANDATORY = (-2, "Event subject is mandatory")
CANNOT_PROCESS_SYNC_EVENT = (-3, "Cannot process sync event")
NOT_A_UUID = (-4, "Definition id is not a valid UUID")
INVALID_UTF8_DATA = (-5, "Event data was not utf-8")
ASYNC_TARGET_MUST_HAVE_REPLY_SUBJECT = (-6, "The asyncTarget node must have a replySubject")
EVENT_MANDATORY = (-7, "Event node is mandatory")
ASYNC_TARGET_MUST_HAVE_DATA = (-8, "The event of an asyncTarget must have data")
ASYNC_TARGET_MUST_HAVE_JSON_DATA = (-9, "Async target must have a json data")
ASYNC_TARGET_MUST_DATA_MUST_CONTAIN_AN_ID = (-10, "Async target data must be a json with an id")
CANNOT_PROCESS_TARGET_AND_ASYNC_TARGET = (-11, "Cannot process target and async target")
