class ApplyCouponErrorCode(object):
    COUPON_NOT_FOUND = "COUPON_NOT_FOUND"
    CUSTOM_ERROR = "CUSTOM_ERROR"
    COUPON_LOCKED = "COUPON_LOCKED"
    COUPON_ALREADY_USED = "COUPON_ALREADY_USED"
    NDISCOUNT_ERROR = "NDISCOUNT_ERROR"
