from typing import Dict, Any

from nreports.builders.pre_account import PreAccountBySeatsBuilder
from nreports_api.shared import JSON


class ConferenceItemsBySeatsBuilder(PreAccountBySeatsBuilder):

    def build_data(
        self,
        data_to_build: Dict[str, JSON],
        **kwargs: Any,
    ) -> Dict[str, JSON]:

        data_to_build = super().build_data(
            data_to_build=data_to_build,
            **kwargs,

        )
        data_to_build["SHOW_ORDER_ID"] = False

        self._build_footer(data_to_build=data_to_build)

        return data_to_build
