from typing import Dict

from ._TableError import TableError


class TabNotInValidTabNames(TableError):

    def __init__(
        self,       # type: TabNotInValidTabNames
        tab_name,   # type: str
    ):
        # type: (...) -> None
        super(TabNotInValidTabNames, self).__init__("$TAB_NOT_REGISTERED|{}".format(tab_name))

        self.tab_name = tab_name

    def get_extra_data(
        self,   # type: TabNotInValidTabNames
    ):
        # type: (...) -> Dict[str, str]

        return {
            "tabName": self.tab_name,
        }
