from helper import ExtendedEnum


# noinspection SpellCheckingInspection
class FiscalPaymentTypes(ExtendedEnum):
    DINHEIRO = ("01", "Dinheiro")
    CHEQUE = ("02", "Cheque")
    CARTAO_CREDITO = ("03", "Cartão de Crédito")
    CARTAO_DEBITO = ("04", "Cartão de Débito")
    CREDITO_LOJA = ("05", "Crédito Loja")
    VALE_ALIMENTACAO = ("10", "Vale Alimentação")
    VALE_REFEICAO = ("11", "Vale Refeição")
    VALE_PRESENTE = ("12", "Vale Presente")
    VALE_COMBUSTIVEL = ("13", "Vale Combustível")
    BOLETO_BANCARIO = ("15", "Boleto Bancário")
    DEPOSITO_BANCARIO = ("16", "Depósito Bancário")
    PIX = ("17", "PIX")
    STATIC_PIX = ("20", "PIX Estático")
    TRANSFERENCIA_BANCARIA = ("18", "Transferência Bancária")
    PROGRAMA_DE_FIDELIDADE = ("19", "Programa de Fidelidade")
    SEM_PAGAMENTO = ("90", "Sem pagamento")
    OUTROS = ("99", "Outros")
