from abc import (
    ABC,
    abstractmethod,
)

from ._EventPublishDetail import EventPublishDetail


class NewEventListener(ABC):
    @abstractmethod
    def new_event(self, publish_detail: EventPublishDetail):
        raise NotImplementedError()
