"""

Revision ID: 8746dd15f3cb
Revises: 37c1f7921b35
Create Date: 2024-04-08 14:42:17.499202

"""
from typing import Sequence, Union

from alembic import op


# revision identifiers, used by Alembic.
revision: str = '8746dd15f3cb'
down_revision: Union[str, None] = '37c1f7921b35'
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    op.create_index(
        index_name="PendingEvent_AckedAt",
        table_name="PendingEvent",
        columns=("AckedAt",),
    )

    op.create_index(
        index_name="PendingEvent_AbandonedAt",
        table_name="PendingEvent",
        columns=("AbandonedAt",),
    )


def downgrade() -> None:
    op.drop_index(
        index_name="PendingEvent_AckedAt",
        table_name="PendingEvent",
    )

    op.drop_index(
        index_name="PendingEvent_AbandonedAt",
        table_name="PendingEvent",
    )
