from messagebus import (
    TokenPriority,
    TokenCreator,
)

_USER_CONTROL_GROUP = "12"

TK_USER_CONTROL_AUTHENTICATE = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group=_USER_CONTROL_GROUP,
    number="1",
)
TK_USER_CONTROL_GET_INFO = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group=_USER_CONTROL_GROUP,
    number="2"
)
TK_USER_CONTROL_GET_USERS_LIST = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group=_USER_CONTROL_GROUP,
    number="3"
)
TK_USER_CONTROL_AUTHENTICATE_NFC = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group=_USER_CONTROL_GROUP,
    number="4",
)
TK_USER_CONTROL_INSERT_OR_UPDATE_USERS = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group=_USER_CONTROL_GROUP,
    number="5",
)
