class TagNotFoundException(Exception):
    pass


class EventNotFoundException(Exception):
    pass


class PrinterNameFoundException(Exception):
    pass


class ViewNameFoundException(Exception):
    pass


class CommandNotFoundException(Exception):
    pass


class BuilderNotFoundException(Exception):
    pass


class ReprocessTimeNotFoundException(Exception):
    pass


class ReprocessInvalidTimeException(Exception):
    pass


class NoBoxesConfiguredException(Exception):
    pass


class NoViewsConfiguredException(Exception):
    pass


class PickListOrderNotFoundException(Exception):
    pass


class InvalidOrderToPrint(Exception):
    pass


class JitNotFoundException(Exception):
    pass


class StatisticClassNotFoundException(Exception):
    pass


class MissingBoxParameterException(Exception):
    pass
